/* ***********************************************************************  
  FREE IP GENERAL PUBLIC LICENSE
  TERMS AND CONDITIONS FOR USE, COPYING, DISTRIBUTION, AND MODIFICATION

  1.  You may copy and distribute verbatim copies of this core, as long
      as this file, and the other associated files, remain intact and
      unmodified.  Modifications are outlined below.  
  2.  You may use this core in any way, be it academic, commercial, or
      military.  Modified or not.  
  3.  Distribution of this file must be free of charge.  Charging is
      allowed only for value added services.  Value added services
      would include copying fees, modifications, customizations, and
      inclusion in other products.
  4.  If a modified source code is distributed, the original unmodified
      source code must also be included (or a link to my web
      site).  In the modified source code there must be clear
      identification of the modified version.
  5.  Visit the my web site for additional information.
      http://www.iosifk.narod.ru

*********************************************************************** */ 

////////////////////////////////////////////////////////////////////////
//	Name File		  : my_jtag_port.v 					    											//
//	Autor					: Iosif Karshenboim																	//
//	      				: www.iosifk.narod.ru																//
//	Description		: JTAG port model                   								//
//	Start design	: 23.08.2006																				//
//	Last revision	: 23.08.2006																				//
////////////////////////////////////////////////////////////////////////
 


`timescale 1ns / 10 ps
 
module my_jtag_port 				 
			 (
        Reset,
// JTAG - port pins
        my_tdi,
        my_tck,
        my_tms,
        my_tdo,
// emulating FPGA shift IR regs
        IrScanData,
// TAP states
        TLR,
        RTI,
        SDR,
        CDR,
        SHDR,
        E1DR,
        PDR,
        E2DR,
        UDR,
        SIR,
        CIR,
        SHIR,
        E1IR,
        PIR,
        E2IR,
        UIR,
// to USER
        user_reset,
        user_tdi,  
        user_tck,  
        user_tdo  
        );

parameter IrWidth = 6;
parameter BYPASS  =6'b111111;
parameter IDCODE  =6'b001001;
parameter USERCODE=6'b001000;
parameter USER1   =6'b000010;
parameter USER2   =6'b000011;
parameter USER3   =6'b000000;
parameter USER4   =6'b000000;

parameter IDCODE_DATA   =32'b00000001110000010000000010010011;
parameter USERCODE_DATA =32'h55;
//parameter IrScan   =6'b110101;



// JTAG - port pins
input		        Reset;
input		        my_tdi;
input		        my_tck;
input		        my_tms;
output	        my_tdo;

// emulating FPGA shift IR regs
input [IrWidth-1:0]  IrScanData;       //  

// TAP states
output          TLR;
output          RTI;
output          SDR;
output          CDR;
output          SHDR;
output          E1DR;
output          PDR;
output          E2DR;
output          UDR;
output          SIR;
output          CIR;
output          SHIR;
output          E1IR;
output          PIR;
output          E2IR;
output          UIR;

// to USER
output          user_reset;
output		      user_tdi;
output	        user_tck;
input	          user_tdo;




reg [IrWidth-1:0] IrScanData_rg;
reg [31:0]        DrScanData_rg;
reg [IrWidth-1:0] Ir_rg;
reg [6:0]         ir_decodes;
wire              user4_en, user3_en, user2_en, user1_en, 
                  user_code_en, id_code_en, bypass_en;
reg				        my_tdo;



wire   [15:0] jtag_state;
reg		 [3:0]  reset_cnt;
reg				    user_reset;
reg				    bypass_rg;

my_tap_states tap_states				 
			       (
              .Reset (Reset),
              // JTAG - port pins
              .my_tck (my_tck),
              .my_tms (my_tms),
              // TAP states
              .tap_states (jtag_state)
              );



assign TLR  = jtag_state[0];     // test_logic_reset
assign RTI  = jtag_state[1];     // run_test_idle
assign SDR  = jtag_state[2];     // sel_dr_scan
assign SIR  = jtag_state[3];     // sel_ir_scan
assign CDR  = jtag_state[4];     // capture_dr
assign CIR  = jtag_state[5];     // capture_ir
assign SHDR = jtag_state[6];     // shift_dr
assign SHIR = jtag_state[7];     // shift_ir
assign E1DR = jtag_state[8];     // exit1_dr
assign E1IR = jtag_state[9];     // exit1_ir
assign PDR  = jtag_state[10];    // pause_dr
assign PIR  = jtag_state[11];    // pause_ir
assign E2DR = jtag_state[12];    // exit2_dr
assign E2IR = jtag_state[13];    // exit2_ir
assign UDR  = jtag_state[14];    // update_dr
assign UIR  = jtag_state[15];    // update_ir


assign user_tck   = my_tck;
assign user_tdi   = my_tdi;


//--------------------------------------------------
// Reset counter 
//--------------------------------------------------
	always @(posedge my_tck or posedge Reset)
		if (Reset)
     begin
			reset_cnt    <= 0;
 			user_reset   <= 1'b1;
     end
		else if (~TLR) //  
       begin
			   reset_cnt    <= 0;
 			   user_reset   <= 1'b0;
       end
		else if ((reset_cnt < 5) && TLR)   // Time < 6*my_tck
     begin
			   reset_cnt    <= reset_cnt + 1;
 			   user_reset   <= 1'b0;
     end
		else if ((reset_cnt == 5) && TLR)   // time OK     
     begin
			   reset_cnt    <= reset_cnt;
 			   user_reset   <= 1'b1;
     end

//--------------------------------------------------
// command shift reg 
//--------------------------------------------------
always @(posedge my_tck or posedge Reset)
		if (Reset)
			IrScanData_rg[IrWidth-1:0] <= IrScanData[IrWidth-1:0];
		else if (SHIR)
       begin
        IrScanData_rg[IrWidth-2:0] <= IrScanData_rg[IrWidth-1:1];   // shift ->
        IrScanData_rg[IrWidth-1]   <= my_tdi;
       end
		else if (E1IR | UIR)
     IrScanData_rg[IrWidth-1:0] <= IrScanData_rg[IrWidth-1:0];
    else
     IrScanData_rg[IrWidth-1:0] <= IrScanData[IrWidth-1:0];

//--------------------------------------------------
// command hold reg 
//--------------------------------------------------
always @(posedge my_tck or posedge Reset)
		if (Reset)
			Ir_rg <= 0;
		else if (UIR)
      Ir_rg <= IrScanData_rg;   // load
		else
      Ir_rg <= Ir_rg;

//--------------------------------------------------
// command decoder 
//--------------------------------------------------

assign {user4_en, user3_en, user2_en, user1_en, user_code_en, id_code_en, bypass_en} = ir_decodes;
always @(Ir_rg) 
 begin
	casex (Ir_rg)								 // synopsys parallel_case
                               // For reference, the Ir commands are: 
      BYPASS   : ir_decodes = 7'b 0_0_0_0_0_0_1; // 1 - bypass_en
      IDCODE   : ir_decodes = 7'b 0_0_0_0_0_1_0; // 2 - id_code_en
      USERCODE : ir_decodes = 7'b 0_0_0_0_1_0_0; // 3 - user_code_en
      USER1    : ir_decodes = 7'b 0_0_0_1_0_0_0; // 4 - user1_en
      USER2    : ir_decodes = 7'b 0_0_1_0_0_0_0; // 5 - user2_en
      USER3    : ir_decodes = 7'b 0_1_0_0_0_0_0; // 6 - user3_en
      USER4    : ir_decodes = 7'b 1_0_0_0_0_0_0; // 7 - user4_en
                                                                                   
    default:
			ir_decodes = 7'b 0_0_0_0_0_0_0;
	endcase
 end

//--------------------------------------------------
// data shift reg 
//--------------------------------------------------
always @(posedge my_tck or posedge Reset)
		if (Reset)
      DrScanData_rg[31:0] <= 0;
		
		else if (CDR & id_code_en)                             
        DrScanData_rg[31:0] <= IDCODE_DATA;      //  
		
		else if (CDR & user_code_en)                             
        DrScanData_rg[31:0] <= USERCODE_DATA;    //  

		else if (SHDR)                                      //  
       begin
        DrScanData_rg[30:0] <= DrScanData_rg[31:1];   // shift ->
        DrScanData_rg[31]   <= my_tdi;
       end
		
		else
      DrScanData_rg[31:0] <= DrScanData_rg[31:0];

//--------------------------------------------------
// Bypass reg 
//--------------------------------------------------
always @(posedge my_tck or posedge Reset)
		if (Reset)
			bypass_rg <= 0;
		else
      bypass_rg <= my_tdi;

//--------------------------------------------------
// TDO data 
//--------------------------------------------------

always @(*)
		if (Reset)
			my_tdo <= 1'b0;    // or may be 1
		else if (SHIR)
       my_tdo <= IrScanData_rg[0];
		else if (SHDR & (id_code_en | user_code_en))
       my_tdo <= DrScanData_rg[0];
    else if (SHDR & ~id_code_en & ~user_code_en)
       my_tdo <= user_tdo;
    else 
       my_tdo <= 1'b0;    // or may be 1

endmodule 






