/* ***********************************************************************  
  FREE IP GENERAL PUBLIC LICENSE
  TERMS AND CONDITIONS FOR USE, COPYING, DISTRIBUTION, AND MODIFICATION

  1.  You may copy and distribute verbatim copies of this core, as long
      as this file, and the other associated files, remain intact and
      unmodified.  Modifications are outlined below.  
  2.  You may use this core in any way, be it academic, commercial, or
      military.  Modified or not. This source is distributed in the hope
			that it will be useful, but WITHOUT ANY WARRANTY; without even the
			implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
			PURPOSE.  See the GNU Lesser General Public License for more
			details.
			
  3.  Distribution of this file must be free of charge.  Charging is
      allowed only for value added services.  Value added services
      would include copying fees, modifications, customizations, and
      inclusion in other products.
  4.  If a modified source code is distributed, the original unmodified
      source code must also be included (or a link to my web
      site).  In the modified source code there must be clear
      identification of the modified version.
  5.  Visit the my web site for additional information.
      http://www.iosifk.narod.ru

*********************************************************************** */ 
////////////////////////////////////////////////////////////////////////
//	Name File		  : tb.v 					              											//
//	Autor					: Iosif Karshenboim																	//
//	      				: www.iosifk.narod.ru																//
//	Description		: my_tap_states                     								//
//	Start design	: 23.08.2006																				//
//	Last revision	: 23.08.2006																				//
////////////////////////////////////////////////////////////////////////

`timescale 1ns / 10 ps

module my_tap_states 				 
			 (
        Reset,
// JTAG - port pins
        my_tck,
        my_tms,
// TAP states
        tap_states
        );

// JTAG - port pins
input		        Reset;
input		        my_tck;
input		        my_tms;

// TAP states
output [15:0] tap_states;


parameter test_logic_reset  = 16'b0000000000000001;
parameter run_test_idle     = 16'b0000000000000010;
parameter sel_dr_scan       = 16'b0000000000000100;
parameter sel_ir_scan       = 16'b0000000000001000;
parameter capture_dr        = 16'b0000000000010000;
parameter capture_ir        = 16'b0000000000100000;
parameter shift_dr          = 16'b0000000001000000;
parameter shift_ir          = 16'b0000000010000000;
parameter exit1_dr          = 16'b0000000100000000;
parameter exit1_ir          = 16'b0000001000000000;
parameter pause_dr          = 16'b0000010000000000;
parameter pause_ir          = 16'b0000100000000000;
parameter exit2_dr          = 16'b0001000000000000;
parameter exit2_ir          = 16'b0010000000000000;
parameter update_dr         = 16'b0100000000000000;
parameter update_ir         = 16'b1000000000000000;

//parameter test_logic_reset  = 0,  run_test_idle  = 1,  sel_dr_scan  = 2,  sel_ir_scan  = 3,
//          capture_dr  = 4,        capture_ir  = 5,     shift_dr = 6,      shift_ir = 7,
//          exit1_dr = 8,           exit1_ir = 9,        pause_dr  = 10,    pause_ir  = 11,
//          exit2_dr = 12,          exit2_ir = 13,       update_dr  = 14,   update_ir  = 15;


reg   [15:0]      jtag_state = test_logic_reset;

assign tap_states = jtag_state;

   always@(posedge my_tck) begin
      if (Reset) 
         jtag_state <= test_logic_reset;
      else
         case (jtag_state) //(* FULL_CASE, PARALLEL_CASE *)
            test_logic_reset : begin
               if (my_tms) 
					       jtag_state <= test_logic_reset;							// 
				        else 
					       jtag_state <= run_test_idle;	
            end
            run_test_idle : begin
               if (my_tms) 
					       jtag_state <= sel_dr_scan;							// 
				        else 
					       jtag_state <= run_test_idle;
               end
            sel_dr_scan : begin
			         if (my_tms) 
					       jtag_state <= sel_ir_scan;							// 
				       else 
					       jtag_state <= capture_dr;
               end  
            sel_ir_scan : begin
               if (my_tms)
                 jtag_state <= test_logic_reset;							// 
				       else 
					       jtag_state <= capture_ir;			 	
               end
            capture_dr : begin
               if (my_tms) 
					       jtag_state <= exit1_dr;							// 
				       else 
					       jtag_state <= shift_dr;			 	
			         end
            capture_ir : begin
                if (my_tms) 
					        jtag_state <= exit1_ir;							// 
				        else 
					        jtag_state <= shift_ir;			 	
			          end
            shift_dr : begin
               if (my_tms) 
					        jtag_state <= exit1_dr;							// 
				       else 
					        jtag_state <= shift_dr;			 	
			         end
            shift_ir : begin
               	if (my_tms) 
					        jtag_state <= exit1_ir;							// 
				        else 
					        jtag_state <= shift_ir;			 	
			          end
            exit1_dr : begin
               if (my_tms)
                  jtag_state <= update_dr;							// 
				       else 
					        jtag_state <= pause_dr;			 	
			         end
            exit1_ir : begin
               if (my_tms)
                  jtag_state <= update_ir;							// 
				       else 
					        jtag_state <= pause_ir;			 	
			         end
            pause_dr : begin
               if (my_tms)
                  jtag_state <= exit2_dr;							// 
				       else 
					        jtag_state <= pause_dr;			 	
			         end
            pause_ir : begin
               if (my_tms)
                  jtag_state <= update_ir;							// 
				        else 
					        jtag_state <= pause_ir;			 	
			         end
            exit2_dr : begin
               if (my_tms)
                  jtag_state <= update_dr;							// 
				       else 
					        jtag_state <= shift_dr;			 	
			         end
            exit2_ir : begin
               if (my_tms)
					        jtag_state <= update_ir;							// 
				       else 
					        jtag_state <= shift_ir;			 	
			         end
            update_dr : begin
               if (my_tms)
                  jtag_state <= sel_dr_scan;							// 
				       else 
					        jtag_state <= run_test_idle;			 	
			         end
            update_ir : begin
               if (my_tms)
					        jtag_state <= sel_dr_scan;							// 
				       else 
					        jtag_state <= run_test_idle;			 	
			         end
         endcase
   end

endmodule 





