/* ***********************************************************************  
  FREE IP GENERAL PUBLIC LICENSE
  TERMS AND CONDITIONS FOR USE, COPYING, DISTRIBUTION, AND MODIFICATION

  1.  You may copy and distribute verbatim copies of this core, as long
      as this file, and the other associated files, remain intact and
      unmodified.  Modifications are outlined below.  
  2.  You may use this core in any way, be it academic, commercial, or
      military.  Modified or not. This source is distributed in the hope
			that it will be useful, but WITHOUT ANY WARRANTY; without even the
			implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
			PURPOSE.  See the GNU Lesser General Public License for more
			details.
			
  3.  Distribution of this file must be free of charge.  Charging is
      allowed only for value added services.  Value added services
      would include copying fees, modifications, customizations, and
      inclusion in other products.
  4.  If a modified source code is distributed, the original unmodified
      source code must also be included (or a link to my web
      site).  In the modified source code there must be clear
      identification of the modified version.
  5.  Visit the my web site for additional information.
      http://www.iosifk.narod.ru

*********************************************************************** */ 
////////////////////////////////////////////////////////////////////////
//	Name File		  : tb.v 					              											//
//	Autor					: Iosif Karshenboim																	//
//	      				: www.iosifk.narod.ru																//
//	Description		: JTAG port model                   								//
//	Start design	: 23.08.2006																				//
//	Last revision	: 23.08.2006																				//
////////////////////////////////////////////////////////////////////////


//`include "timescale.v"
`include "my_jtag_port.v"

`timescale 1ns/10ps

module tb_my_top();

parameter Width = 32;

reg           StartTB;
reg           CLK;
reg           RST;
reg           HostBusy;
reg           TDI, TMS, TCK;
wire          wTDO;

//--------------------------
// Connecting plug module

//plug myplug  // plug
//			 (
//				.RST (RST),
//				.TCK (TCK),
//				.TDI (TDI),   //  
//				.TMS (TMS),
//				.TDO (wTDO)   // 
//				);

my_jtag_port dut				 
			 (
        .Reset (RST),
// JTAG - port pins
        .my_tdi (TDI),     // 
        .my_tck (TCK),
        .my_tms (TMS),
        .my_tdo (wTDO),   // 
// emulating FPGA shift IR regs
        .IrScanData (6'b110101),
// TAP states
        .TLR (),
        .RTI (),
        .SDR (),
        .CDR (),
        .SHDR (),
        .E1DR (),
        .PDR (),
        .E2DR (),
        .UDR (),
        .SIR (),
        .CIR (),
        .SHIR (),
        .E1IR (),
        .PIR (),
        .E2IR (),
        .UIR (),
// to USER
        .user_reset (),
        .user_tdi (),    
        .user_tck (),  
        .user_tdo (1'b0)  // input
        );


//--------------------------
initial
// Generating CLK clock
begin
  forever #50 CLK = ~CLK;  // 2*50 ns -> 10 MHz    
end

initial
begin
           CLK  =  1'b0;
       HostBusy =  1'b0;
       StartTB  =  1'b0;

            TDI = 1'b0;
            TMS = 1'b0;
            TCK = 1'b0;

	          RST =  1'b1;  // Reset pulse ON
  #100      RST =  1'b0;  // Reset pulse OFF
  #100 StartTB  =  1'b1;
  
end

initial
begin
  $display ("Host for JTAG 1.0.  This is the begginning of the TEST.");
  wait(StartTB);  // Start of testbench

//  tester(32'hC3FF, 32'hC355, 8, wTDO, RST);     //
//  tester(32'hC35B, 32'hC355, 8, wTDO, RST);     //
//  tester(32'hC35D, 32'hC35B, 4, wTDO, RST);     // 
//  tester(32'hC35D, 32'hC35B, 1, wTDO, RST);     // 

// Get ID    
//  TAP_TLR, tms - 0x1F, tdi - 0x00, N - 0x05
tester(32'h00FF, 32'h0000, 8, wTDO, RST);

//  TAP_SI, tms - 0x06, tdi - 0x00, N - 0x05
tester(32'h0006, 32'h0000, 5, wTDO, RST);

//    , tms - 0x00, tdi - 0x09, N - 0x05, RdWriteTMS_SetTDI strScanIn - 10101
tester(32'h0000, 32'h0009, 5, wTDO, RST);

//       TapEndState, tms - 0x0B, tdi - 0x00, N - 0x05, RdWriteTMS_SetTDI strScanIn - 11111
tester(32'h000B, 32'h0000, 5, wTDO, RST);

//cur_state - 0x03
//  - 0x02
//String IrScan - 110101
//   , tms - 0x00, tdi - 0x00, N - 0x08, RdWriteTMS_SetTDI strScanIn - 00100111
tester(32'h0000, 32'h0000, 8, wTDO, RST);
//   , tms - 0x00, tdi - 0x00, N - 0x08, RdWriteTMS_SetTDI strScanIn - 00000001
//   , tms - 0x00, tdi - 0x00, N - 0x08, RdWriteTMS_SetTDI strScanIn - 10000010
//   , tms - 0x00, tdi - 0x00, N - 0x08, RdWriteTMS_SetTDI strScanIn - 00000011   ???????????????
//    , tms - 0x00, tdi - 0x00, N - 0x00, RdWriteTMS_SetTDI strScanIn -   ???????????????
//      TapEndState, tms - 0x03, tdi - 0x00, N - 0x03, RdWriteTMS_SetTDI strScanIn - 110
tester(32'h0003, 32'h0000, 3, wTDO, RST);
//cur_state - 0x08
//  - 0x01






	#1000 $stop;
		 
end // initial



task tester;

parameter Width = 32;

input [Width-1:0] TMS_Data;			//
input [Width-1:0] TDI_Data;			//
input [Width-1:0] bits_number;	//  1, 2, 3, .... bits to shift
input             TDO_Data;			// 
input		           RST;


reg		[Width-1:0] TDO_reg;
reg		[Width-1:0] TDI_reg;
reg		[Width-1:0] TMS_reg;
reg			          tap_clk;

integer      i, j, k;

	
begin

$display ("Host for JTAG 1.0.  This is the BASIC TEST.");

j = bits_number;     //  1, 2, 3, ....


@ (posedge CLK);
		if (RST)
       begin
			TDO_reg  <= 0;
         TDI_reg  <= 0;
         TMS_reg  <= 0;
         HostBusy <= 1'b0;   //
         tap_clk  <= 1'b0; 
       end
		else   
       begin
         TDO_reg  <= 0;
         TDI_reg  <= TDI_Data;
         TMS_reg  <= TMS_Data;
         HostBusy <= 1'b1;   //
         tap_clk  <= 1'b0;
       end

      i = 0;
      k = j;
      while (i <= (j * 2)+1) 
       begin
        @ (posedge CLK);
          if (k != 0)        // time is in progress     Width-1:1     
            begin
             if(tap_clk)
               begin
                tap_clk  <= 1'b0;
                k = k - 1;	                                // decrement(count)
                TDO_reg[Width-2:0] <= TDO_reg[Width-1:1];   // shift ->
			       //   TDO_reg[Width-1]   <= TDO_Data;             // serial data input from TDO
                TDO_reg[Width-1]   <= wTDO;             // serial data input from TDO

                TDI_reg[Width-2:0] <= TDI_reg[Width-1:1];   // shift ->
			          TDI_reg[Width-1]   <= 1'b0;
								TMS_reg[Width-2:0] <= TMS_reg[Width-1:1];   // shift ->
			          TMS_reg[Width-1]   <= 1'b0;
                $display ("Tdo - %0b, Tdi - , %0b, Tms -  %0b.", wTDO, TDI_reg[0], TMS_reg[0]);
               end
             else
               tap_clk  <= 1'b1;
      
             i = i + 1;
            end

		      else if (k == 0)   // time OK         
            begin
             tap_clk  <= 1'b0;
             HostBusy <= 1'b0;       //
             TDO_reg  <= TDO_reg;
             //TDI_reg  <= TDI_reg;
             //TMS_reg  <= TMS_reg;
             TDI_reg[0]  <= 1'b0;
             TMS_reg[0]  <= 1'b0;
             
             i = i + 1;
            end

         TDI <= TDI_reg[0];
         TMS <= TMS_reg[0];
         TCK <= tap_clk;

      end  

$display ("Tdo_reg - %0h", TDO_reg);
 end 
endtask


endmodule
